import math
import imp
imp.load_source("util","ScriptEditor/util.py")  
from util import *
import sys


translator.install()

# Get the most appropriate temperature couple to use for factors computation
def getLGAirTemperatures(boundaryConditions, boundaryGroups):
	maxAirTemperature = -10000.0
	for boundaryGroup in boundaryGroups:
		for boundaryID in boundaryGroup:
			if "temperature" in boundaryConditions[boundaryID]["flows"] and \
				not isinstance(boundaryConditions[boundaryID]["flows"]["temperature"], str) and \
				boundaryConditions[boundaryID]["flows"]["temperature"] > maxAirTemperature:
				maxAirTemperature = boundaryConditions[boundaryID]["flows"]["temperature"]
			if "air_temperature" in boundaryConditions[boundaryID]["flows"] and \
				not isinstance(boundaryConditions[boundaryID]["flows"]["air_temperature"], str) and \
				boundaryConditions[boundaryID]["flows"]["air_temperature"] > maxAirTemperature:
				maxAirTemperature = boundaryConditions[boundaryID]["flows"]["air_temperature"]
	lgAirTemperature1 = maxAirTemperature
	lgAirTemperature2 = 0.0
	if lgAirTemperature1 <= 0.0 or lgAirTemperature1 > 10000.0 or math.fabs(lgAirTemperature1 - lgAirTemperature2) < 1.0:
		lgAirTemperature1 = 1.0
		lgAirTemperature2 = 0.0
	return [lgAirTemperature1, lgAirTemperature2]

# Generate a table representing which boundary groups must be set to a particular temperature
# for a specific computation
def generateLResultsGrid(size):
	if size == 2:
		return [[1, 0], [0, 1]]
	elif size == 3:
		return [[1, 0, 0], [0, 1, 0], [0, 0, 1]]
	elif size == 4:
		return [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1], [1, 1, 0, 0], [1, 0, 1, 0]]
	else:
		return list()

def createLResultsTableDescriptor(boundaryGroups, simulation_3d):
	if not simulation_3d:
		unit = "W/mK"
	else:
		unit = "W/K"
		
	lComputationsDescriptor = createEmptyGridDescriptor("l_computations", _("Temperatures used for L coefficients"), "l_computations", 600, -1)
	addGridColStringDescriptor(lComputationsDescriptor, "name", _("Name"))
	for boundaryGroupID in range(len(boundaryGroups)):
		addGridColNumberDescriptor(lComputationsDescriptor, "t" + str(boundaryGroupID), "T" + str(boundaryGroupID + 1), "°C", "%.2f")
	addGridColNumberDescriptor(lComputationsDescriptor, "result", _("Result"),  unit, "%.4f")
	return lComputationsDescriptor
	
# Compute L results
def computeLResultsTable(simulator,\
						currentGeometry,\
						currentFill,\
						currentBoundary,\
						flowSimulation,\
						Input,\
						Output,\
						OutputDescriptors,\
						lgAirTemperature1,\
						lgAirTemperature2,\
						lComputationsGrid,\
						boundaryGroups):
						
	simulationSettings = simulator.getSimulationSettings(currentGeometry, currentFill, currentBoundary)
	discardL = True
	if "flows" in simulationSettings:
		if "discard_l" in simulationSettings["flows"]:
			discardL = simulationSettings["flows"]["discard_l"]
	
	lComputationsResult = list()
	specialElementsConfig = list()
	for computationsRow in range(len(lComputationsGrid)):
		lComputationsResult.append(dict())

	boundaryVariant = "flows"
	lInput = dict(Input)
	lInput["boundary_variant"] = boundaryVariant
	#lInput["discard_results"] = discardL
	lInput["progress_bar_multiplier"] = len(lComputationsGrid)
	specialElementsConfig = list()
	for computationsRow in range(len(lComputationsGrid)):
		
		lComputationsResult[computationsRow]["name"] = "Computation" + " " + str(computationsRow + 1)

		computationBoundaryConditions = simulator.getBoundaryConditions(currentGeometry, currentFill, currentBoundary)

		for computationsCol in range(len(lComputationsGrid[computationsRow])):
			# Setup all the boundary conditions for this computation
			if lComputationsGrid[computationsRow][computationsCol] == 1:
				groupTemperature = lgAirTemperature1
			else:
				groupTemperature = lgAirTemperature2
			lComputationsResult[computationsRow]["t" + str(computationsCol)] = groupTemperature

			for boundaryID in boundaryGroups[computationsCol]:
				computationBoundaryCondition = computationBoundaryConditions[boundaryID][boundaryVariant]

				tfAnalysis = flowSimulation["advanced_results"]["boundary"]["tf_analysis"]					
				boundariesAtSimulationTime = flowSimulation["advanced_results"]["boundary"]["boundaries_at_simulation_time"]
				if boundaryID < len(boundariesAtSimulationTime) and boundaryID < len(tfAnalysis):
					computationBoundaryCondition["surface_resistance_ta"] = boundariesAtSimulationTime[boundaryID]["temperature"]
					computationBoundaryCondition["surface_resistance_ts"] = tfAnalysis[boundaryID]["av_t"]
				
				computationBoundaryCondition["base"] = "none"
				computationBoundaryCondition["air_temperature_type"] = "fixed"
				computationBoundaryCondition["air_temperature"] = groupTemperature
				computationBoundaryCondition["temperature_type"] = "fixed"
				computationBoundaryCondition["temperature"] = groupTemperature

		# Perform a simulation with new boundary conditions
		if discardL:
			simulationOutput, simulationOutputDescriptors = simulator.simulate(currentGeometry,\
																				currentFill,\
																				currentBoundary,\
																				specialElementsConfig,\
																				computationBoundaryConditions,\
																				lInput)
		else:
			simulationOutput, simulationOutputDescriptors = addSimulation(simulator,\
																			currentGeometry,\
																			currentFill,\
																			currentBoundary,\
																			specialElementsConfig,\
																			computationBoundaryConditions,\
																			lInput,\
																			Output,\
																			OutputDescriptors,\
																			"L " + _("Computation") + " " + str(computationsRow + 1),\
																			"lc_" + str(computationsRow + 1) + "_s")															
																		
		lInput["progress_bar_multiplier"] = 0

		# Compute heat flow
		simulation_3d = "3d_simulation" in simulationOutput and simulationOutput["3d_simulation"]
		if simulation_3d:
			heatFlow = getAvFlow3D(simulationOutput["advanced_results"]["boundary"]["tf_analysis"], computationBoundaryConditions)
		else:
			heatFlow = getAvFlow(simulationOutput["advanced_results"]["boundary"]["tf_analysis"])

		lComputationsResult[computationsRow]["result"] = heatFlow / math.fabs(lgAirTemperature1 - lgAirTemperature2)

		if len(lComputationsGrid) == 2:
			lComputationsResult[1]["name"] = "Computation" + " 2"
			lComputationsResult[1]["t0"]  = lgAirTemperature2
			lComputationsResult[1]["t1"]  = lgAirTemperature1
			lComputationsResult[1]["result"] = lComputationsResult[0]["result"]
			break
			
	return lComputationsResult
		
def createLTableDescriptor(boundaryGroupNames, simulation_3d):
	lTableDescriptor = createEmptyGridDescriptor("l_table", _("L - thermal coupling coefficients"), "l_table", 600, -1)
	addGridColStringDescriptor(lTableDescriptor, "name", "");
	if not simulation_3d:
		unit = "W/mK"
	else:
		unit = "W/K"
		
	for boundaryGroupID in range(len(boundaryGroupNames)):
		addGridColNumberDescriptor(lTableDescriptor,  str(boundaryGroupID), boundaryGroupNames[boundaryGroupID], unit, "%.4f");
	return lTableDescriptor

# Compute L table from an l computation grid
def computeLTable(boundaryGroupNames, lComputationsResult):
	lTableResults = list()
	for boundaryGroupID in range(len(boundaryGroupNames)):
		lTableResults.append({"name" : boundaryGroupNames[boundaryGroupID]})

	lgComputationsResults = list()
	for lgComputationResult in lComputationsResult:
		lgComputationsResults.append(lgComputationResult["result"])

	if len(boundaryGroupNames) == 2:

		lTableResults[0]["0"] = lTableResults[1]["1"] = 0.0
		lTableResults[0]["1"] = lTableResults[1]["0"] = lgComputationsResults[0]

	elif len(boundaryGroupNames) == 3:

		lTableResults[0]["0"] = lTableResults[1]["1"] = lTableResults[2]["2"] = 0.0
		lTableResults[0]["1"] = lTableResults[1]["0"] = 0.5 * (lgComputationsResults[0] + lgComputationsResults[1] - lgComputationsResults[2])
		lTableResults[0]["2"] = lTableResults[2]["0"] = 0.5 * (lgComputationsResults[0] - lgComputationsResults[1] + lgComputationsResults[2])
		lTableResults[1]["2"] = lTableResults[2]["1"] = 0.5 * (-lgComputationsResults[0] + lgComputationsResults[1] + lgComputationsResults[2])

	elif len(boundaryGroupNames) == 4:

		lTableResults[0]["0"] = lTableResults[1]["1"] = lTableResults[2]["2"] = lTableResults[3]["3"] = 0.0
		lTableResults[0]["1"] = lTableResults[1]["0"] = 0.5 * (lgComputationsResults[0] + lgComputationsResults[1] - lgComputationsResults[4])
		lTableResults[0]["2"] = lTableResults[2]["0"] = 0.5 * (lgComputationsResults[0] + lgComputationsResults[2] - lgComputationsResults[5])
		lTableResults[0]["3"] = lTableResults[3]["0"] = lgComputationsResults[0]- lTableResults[0]["1"] - lTableResults[0]["2"]
		lTableResults[2]["3"] = lTableResults[3]["2"] = 0.5 * (lgComputationsResults[2] + lgComputationsResults[3] - lgComputationsResults[4])
		lTableResults[1]["3"] = lTableResults[3]["1"] = 0.5 * (lgComputationsResults[0] + lgComputationsResults[3] - lgComputationsResults[5])
		lTableResults[1]["2"] = lTableResults[2]["1"] = lgComputationsResults[1] - lTableResults[1]["0"] - lTableResults[1]["3"]

	return lTableResults	

def createFlowTableDescriptor(boundaryGroupNames, simulation_3d):
	if not simulation_3d:
		unit = "W/m"
	else:
		unit = "W"
		
	flowTableDescriptor = createEmptyGridDescriptor("flow_table", _("Flows"), "flow_table", 600, -1)
	addGridColStringDescriptor(flowTableDescriptor, "name", "");
	for boundaryGroupID in range(len(boundaryGroupNames)):
		addGridColNumberDescriptor(flowTableDescriptor,  str(boundaryGroupID), boundaryGroupNames[boundaryGroupID], unit, "%.4f");
	return flowTableDescriptor
	
# Compute flows table
def computeFlowsTable(boundaryResults, boundaryConditions, boundaryGroups):
	lTable = boundaryResults["l_table"]
	boundariesAtSimulationTime = boundaryResults["boundaries_at_simulation_time"]
	flowTable = list()
	row = 0
	for lTableRow in lTable:
		rowBoundaryID = boundaryConditions[boundaryGroups[row][0]]["id"]
		flowTableRow = dict()
		for lTableItemKey, lTableItemValue in lTableRow.items():
			if lTableItemKey != "name":			
				col = int(lTableItemKey)
				colBoundaryID = boundaryConditions[boundaryGroups[col][0]]["id"]
				deltaT = boundariesAtSimulationTime[rowBoundaryID]["temperature"] - boundariesAtSimulationTime[colBoundaryID]["temperature"]
				flowTableRow[lTableItemKey] = math.fabs(lTableItemValue * deltaT)
			else:
				flowTableRow[lTableItemKey] = lTableItemValue
		flowTable.append(flowTableRow)
		row = row + 1
	return flowTable
	
def computeGTable(simulator,\
					currentGeometry,\
					currentFill,\
					currentBoundary,\
					condensationSimulation,\
					Input,\
					Output,\
					OutputDescriptors,\
					boundaryGroups,\
					boundaryGroupNames):
					
	simulation_3d = "3d_simulation" in condensationSimulation and condensationSimulation["3d_simulation"]

	simulationSettings = simulator.getSimulationSettings(currentGeometry, currentFill, currentBoundary)
	discardG = True
	if "flows" in simulationSettings:
		if "discard_g" in simulationSettings["flows"]:
			discardG = simulationSettings["flows"]["discard_g"]
			
	# Get min temperature point for every boundary group
	condensationAdvanced = condensationSimulation["advanced_results"]
	condensationBoundary = condensationAdvanced["boundary"]
	condensationBoundaryTFAnalysis = condensationBoundary["tf_analysis"]

	boundaryGroupsMinTPos = list()
	for boundaryGroupID in range(len(boundaryGroups)):
		boundaryGroup = boundaryGroups[boundaryGroupID]
		groupMinT = sys.float_info.max
		groupMinTPos = list()
		groupMinTPos.append(0.0)
		groupMinTPos.append(0.0)
		for boundaryID in boundaryGroup:
			boundaryMinT = condensationBoundaryTFAnalysis[boundaryID]["min_t"]
			if boundaryMinT < groupMinT:
				boundaryMinT = groupMinT
				groupMinTPos = condensationBoundaryTFAnalysis[boundaryID]["min_t_pos"]
		boundaryGroupsMinTPos.append(groupMinTPos)
			
	gTable = list()
	specialElementsConfig = list()
	for computationsRow in range(len(boundaryGroups)):
		gTable.append(dict())

	boundaryVariant = "mold"
	gInput = dict(Input)
	gInput["progress_bar_multiplier"] = len(boundaryGroups) - 1
	gInput["boundary_variant"] = boundaryVariant
	gInput["discard_results"] = discardG
	specialElementsConfig = list()
	for computationsRow in range(len(boundaryGroups) - 1):
		computationBoundaryConditions = simulator.getBoundaryConditions(currentGeometry, currentFill, currentBoundary)

		for computationsCol in range(len(boundaryGroups)):
			# Setup all the boundary conditions for this computation
			if computationsCol == computationsRow:
				groupTemperature = 1.0
			else:
				groupTemperature = 0.0

			for boundaryID in boundaryGroups[computationsCol]:
				computationBoundaryConditions[boundaryID][boundaryVariant]["base"] = "none"
				computationBoundaryConditions[boundaryID][boundaryVariant]["air_temperature_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["air_temperature"] = groupTemperature
				computationBoundaryConditions[boundaryID][boundaryVariant]["temperature_type"] = "fixed"
				computationBoundaryConditions[boundaryID][boundaryVariant]["temperature"] = groupTemperature

		# Perform a simulation with new boundary conditions
		if discardG:
			simulationOutput, simulationOutputDescriptors = simulator.simulate(currentGeometry,\
																				currentFill,\
																				currentBoundary,\
																				specialElementsConfig,\
																				computationBoundaryConditions,\
																				gInput)
		else:
			simulationOutput, simulationOutputDescriptors = addSimulation(simulator,\
																			currentGeometry,\
																			currentFill,\
																			currentBoundary,\
																			specialElementsConfig,\
																			computationBoundaryConditions,\
																			gInput,\
																			Output,\
																			OutputDescriptors,\
																			"g " + _("Computation") + " " + str(computationsRow + 1),\
																			"gc_" + str(computationsRow + 1) + "_s")

		Input["progress_bar_multiplier"] = 0
		
		for computationsCol in range(len(boundaryGroups)):
			if not simulation_3d:
				gTable[computationsCol][str(computationsRow)] = simulator.sampleTemperature(boundaryGroupsMinTPos[computationsCol][0],\
																							boundaryGroupsMinTPos[computationsCol][1])
			else:
				gTable[computationsCol][str(computationsRow)] = simulator.sampleTemperature(boundaryGroupsMinTPos[computationsCol][0],\
																							boundaryGroupsMinTPos[computationsCol][1],\
																							boundaryGroupsMinTPos[computationsCol][2])
																							
	for computationsRow in range(len(boundaryGroups)):
		gTable[computationsRow]["name"] = boundaryGroupNames[computationsRow]
		strCol = str(len(boundaryGroups) - 1)
		gTable[computationsRow][strCol] = 1.0
		for computationsCol2 in range(len(boundaryGroups) - 1):
			strCol2 = str(computationsCol2)
			gTable[computationsRow][strCol] = gTable[computationsRow][strCol] - gTable[computationsRow][strCol2]

	return gTable
	
def createGTableDescriptor(boundaryGroupNames):
	gTableDescriptor = createEmptyGridDescriptor("g_table", _("g - temperature weighting factors (on min. temp. points)"), "g_table", 600, -1)
	addGridColStringDescriptor(gTableDescriptor, "name", "");
	for boundaryGroupID in range(len(boundaryGroupNames)):
		addGridColNumberDescriptor(gTableDescriptor,  str(boundaryGroupID), boundaryGroupNames[boundaryGroupID], "", "%.4f");
	return gTableDescriptor
	